/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2020                               **
**                                                                        **
**                            DHCP example                                **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2020.11.26                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.H>
#include "system.h"
#include "types.h"
#include "systime.h"
#include "irqvectors.h"
#include "serialconst.h"
#include "uart0.h"
#include "strings.h"
#include "emacdriver.h"
#include "ipservice.h"
#include "udpservice.h"
#include "dhcpservice.h"
#include "trace.h"

#define IRQCallCntLimit                 500

typedef struct {
                 EMACInstanceRecT *          EMACLayer ;
                 IPLayerInstanceRecT *       IPLayer ;
                 PtrUDPLayerInstanceRecT     UDPLayer ;
                 PtrDHCPLayerInstanceRecT    DHCPLayer ;
                 PtrUDPSocketDescriptionRecT UDPSocket ;
                 volatile USHORT             DHCPReady ;
                 ULONG                       MachineIPAddress ;
                 ULONG                       MachineIPSubnet ;
                 ULONG                       MachineGWIPAddress ;
               } UDPExampleInstanceRecT ;
typedef UDPExampleInstanceRecT * PtrUDPExampleInstanceRecT ;

static UDPExampleInstanceRecT UDPDemoInstance ; ;
static ULONG IRQCallCounter ;

#define NetBufferSize                128

#define UDPPortNumber           20001

UCHAR MachineMACAddr [ 6 ]     = { 0x08 , 0x14 , 0x16 , 0x22 , 0x28 , 0x2D } ;

static UCHAR Error1 [ ]        = "\r\nProblem z tworzeniem gniazdka\r\n" ;
static UCHAR Error2 [ ]        = "\r\nProblem z nadaniem numeru gniazdku\r\n" ;


static __irq void T1_IRQHandler ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter ++ ;
  if ( IRQCallCounter >= IRQCallCntLimit )
  {
    IRQCallCounter = 0 ;
  } /* if */ ;
  T1IR = 1 ;
  VICVectAddr = 0 ;
} /* T1_IRQHandler */


static void HardwareInit ( void ) 
{
  /*-------------------------------------------------------------------------*/
  PINSEL10 = 0 ;
  T1MR0 = ( PCLK / 1000 ) - 1 ;
  T1MCR = 3 ;
  T1TCR = 1 ;
  VICVectAddr5 = ( unsigned long ) T1_IRQHandler ;
  VICVectCntl5 = 15 ;
  VICIntEnable = ( 1 << TIMER1_Interrupt ) ;
  UART0InitEnvir ( ) ;
} /* HardwareInit */


static void DHCPNewDataNotify ( PtrUDPSocketDescriptionRecT DHCPSocket ,
                                ULONG                       NewIPAddress ,
                                ULONG                       NewNetMask ,
                                ULONG                       NewGWAddress )
{
  UCHAR Str [ 32 ] ;
  /*-------------------------------------------------------------------------*/
  UDPDemoInstance . MachineIPAddress = NewIPAddress ;
  UDPDemoInstance . MachineIPSubnet = NewNetMask ;
  UDPDemoInstance . MachineGWIPAddress = NewGWAddress ;
  UDPDemoInstance . DHCPReady = TRUE ;
  UART0SendString ( ( UCHAR * ) "\r\nParametry przydzielone przez DHCP:" ) ;
  UART0SendString ( ( UCHAR * ) "\r\nAdres IP=" ) ;
  IPToString ( Str , UDPDemoInstance . MachineIPAddress ) ;
  UART0SendString ( Str ) ;
  UART0SendString ( ( UCHAR * ) "\r\nMaska podsieci=" ) ;
  IPToString ( Str , UDPDemoInstance . MachineIPSubnet ) ;
  UART0SendString ( Str ) ;
  UART0SendString ( ( UCHAR * ) "\r\nAdres bramy domyslnej=" ) ;
  IPToString ( Str , UDPDemoInstance . MachineGWIPAddress ) ;
  UART0SendString ( Str ) ;
  UART0SendString ( ( UCHAR * ) "\r\n************************\r\n" ) ;
} /* DHCPNewDataNotify */


static void StartNet ( PtrUDPExampleInstanceRecT Instance )
{
  EtherSpeedMode EthMode ;
  /*-------------------------------------------------------------------------*/
  EthMode = Mode_AutoNeg ;
//  EthMode = Mode_10MBIT ;
//  EthMode = Mode_100MBIT ;
  Instance -> EMACLayer = InitPHYLayerInstance ( MachineMACAddr , EthMode ) ;
  Instance -> IPLayer = InitIPLayerInstance ( Instance -> EMACLayer ,
                                              ( ULONG ) 0 , ( ULONG ) 0 ) ;
  Instance -> UDPLayer = StandardAddUDPInstance ( Instance -> IPLayer ) ;
  Instance -> DHCPLayer = AllocateDHCPClientInstance ( Instance -> UDPLayer ,
                                                       ( UCHAR * ) "LPC2378" ) ;
  StartDHCPService ( Instance -> DHCPLayer , DHCPNewDataNotify ) ;
  Instance -> DHCPReady = FALSE ;
  while ( ! Instance -> DHCPReady )
  {
    SysTimerPool ( ) ;
    Instance -> IPLayer -> IPLayerPoolService ( Instance -> IPLayer ) ;
  } /* while */ ;
} /* StartNet */


static void SoftwareInit ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter = 0 ;
  UDPDemoInstance . EMACLayer = NIL ;
  UDPDemoInstance . IPLayer = NIL ;
  UDPDemoInstance . UDPLayer = NIL ;
  UDPDemoInstance . UDPSocket = NIL ;
  UDPDemoInstance . DHCPLayer = NIL ;
  UDPDemoInstance . DHCPReady = FALSE ;
  UDPDemoInstance . MachineIPAddress = ( ULONG ) 0 ;
  UDPDemoInstance . MachineIPSubnet = ( ULONG ) 0 ;
  UDPDemoInstance . MachineGWIPAddress = ( ULONG ) 0 ;
} /* SoftwareInit */


static void UDPStreamService ( PtrUDPSocketDescriptionRecT UDPSocket ,
                               ULONG                       SenderIPAddress ,
                               USHORT                      SenderPortNo ,
                               ULONG                       DestIPAddress ,
                               USHORT                      DestPortNo ,
                               void *                      UDPDataBuffer ,
                               USHORT                      UDPDataSize )
{
  UCHAR Buffer [ NetBufferSize ] ;
  USHORT Loop ;
  USHORT EndLoop ;
  UCHAR * InputBuffer ;
  UCHAR * OutputBuffer ;
  /*-------------------------------------------------------------------------*/
  UART0SendString ( ( UCHAR * ) "\r\nPrzyszedl pakiet UDP od IP=" ) ;
  IPToString ( Buffer , SenderIPAddress ) ;
  UART0SendString ( Buffer ) ;
  UART0SendString ( ( UCHAR * ) ", port=" ) ;
  WordToStr ( Buffer , 5 , ' ' , SenderPortNo ) ;
  UART0SendString ( BegOfString ( Buffer ) ) ;
  UART0SendString ( ( UCHAR * ) " odeslano te echo.\r\n" ) ;
  InputBuffer = ( UCHAR * ) UDPDataBuffer ;
  OutputBuffer = Buffer ;
  * OutputBuffer ++ = 'E' ;
  * OutputBuffer ++ = 'c' ;
  * OutputBuffer ++ = 'h' ;
  * OutputBuffer ++ = 'o' ;
  * OutputBuffer ++ = ':' ;
  * OutputBuffer ++ = ' ' ;
  EndLoop = UDPDataSize ;
  if ( EndLoop >= ( NetBufferSize - 8 ) )
    EndLoop = ( NetBufferSize - 8 ) ;
  for ( Loop = 0 ; Loop < EndLoop ; Loop ++ )
    * OutputBuffer ++ = * InputBuffer ++ ;
  * OutputBuffer = 0 ;
  UDPSendData ( UDPSocket , SenderIPAddress , SenderPortNo , ( void * ) Buffer , StrLen ( Buffer ) ) ;
} /* UDPStreamService */


static void Halt ( UCHAR * Message )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Message ) ;
  for ( ; ; )
  {
    NOP ( ) ;
  } /* loop */ ;
} /* Halt */


UCHAR Hello1 [ ]      = "\r\nHello: serwer UDP z konfiguracja poprzez DHCP" ;
UCHAR Hello2 [ ]      = "\r\nTo co wiadomo, to jedynie moj adres MAC=" ;
UCHAR Hello3 [ ]      = "\r\nOczekiwanie na przydzial adresow...\r\n" ;

static void HelloMessage ( void )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Hello1 ) ;
  UART0SendString ( Hello2 ) ;
  TraceMAC ( MachineMACAddr ) ;
  UART0SendString ( Hello3 ) ;
} /* HelloMessage */


int main ( void )
{
  USHORT BindStatus ;
  /*-------------------------------------------------------------------------*/
  SysInit ( ) ;
  InitSysTime ( ) ;
  HardwareInit ( )  ;
  SoftwareInit ( ) ;
  UART0HardwInit( FOSC , N81Mode , 9600 ) ;
  HelloMessage ( ) ;
  StartNet ( & UDPDemoInstance ) ;
  UDPDemoInstance . UDPSocket =  CreateUDPSocket ( UDPDemoInstance . UDPLayer ) ;
  if ( ! UDPDemoInstance . UDPSocket )
    Halt ( Error1 ) ;
  BindStatus = BindUDPSocket ( UDPDemoInstance . UDPSocket , UDPPortNumber ) ;
  if ( BindStatus != UDP_BindPortOK )
  {
    Halt ( Error2 ) ;
  } /* if */ ;
  SetUDPListen ( UDPDemoInstance . UDPSocket , UDPStreamService ) ;
  for ( ; ; )
  {
    SysTimerPool ( ) ;
    UDPDemoInstance . IPLayer -> IPLayerPoolService ( UDPDemoInstance . IPLayer ) ;
  } /* loop */ ;
} /* main */

